<?php
/**
 * Actions class for WP Gallery plugin
 * all Gallery related ajax calls should be handled by actions defined in this class
 *
 * @package	Gallery WP plug-in
 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
 * @copyright Copyright (c) 2008, Picturesurf.org
 */
class PicturesurfGalleryActions
{
	var $parent;
	var $imageUrl;
	/*
	 * class constructor (old style to continue WP php 4 style)
	 * 
	 * @param $GalleryPluginClass
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function PicturesurfGalleryActions (&$GalleryPluginClass)
	{
		if (isset($GalleryPluginClass))
		{
			$this->parent = $GalleryPluginClass;
		} else
		{
			return;
		}
	}
	/*
	 * check and add uploaded image into DB.
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function addImage ()
	{
		$this->parent->_init();
		$json = new PSGalleryServicesJSON();
		$path = $_POST['path'];
		$title = $_POST['name'];
		$extension = array_pop(explode('.', $path));
		$TblImages = $this->parent->wpdb->prefix . "psimagepages";
		$row = $this->parent->wpdb->get_row('SELECT MAX(id) as ID FROM ' . $TblImages . ';');
		$id = (($row && $row->ID)?$row->ID:0)+1;
		if($_POST['stock'])
		{
			$url = $path;
		}
		else
		{
			$fileName = $this->_generateGalleryCode($id) . '.' . $extension;
			if(!rename($path, $this->parent->dataDir . $fileName))
			{
				@unlink($path);
				echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'Oops! We can\'t find the uploaded image, please try again.'));
				exit();
			}
			$url = $this->parent->dataUrl . $fileName;
		}
		$insert = 'INSERT INTO ' . $TblImages . ' SET id = %d, title = %s, image_url = %s, sequence = 0;';
		$this->parent->wpdb->query($this->parent->wpdb->prepare($insert, $id, $title, $url));
		echo 'status = ' . $json->encode(array('status' => 'ok', 'id' => $id, 'url' => $url, 'name' => rawurlencode($title)));
		exit();
	}
	/*
	 * create image thumbnails
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function thumbnailImage ()
	{
		$this->parent->_init();
		require_once ($this->parent->includesDir . 'ImageProcessor.php');
		$ImProc = new ImageProcessor();
		$json = new PSGalleryServicesJSON();
		$id = $_POST['id'];

		$images = $json->decode(stripcslashes(rawurldecode($_POST['images'])));
		if(!$images || count($images) < 1)
		{
			echo $json->encode(array('status' => 'fail' , 'message' => 'Oops! There is an error in the plugin, it didn\'t send an image.'));
		}
		$processed = array();
		foreach ($images as $img)
		{
			$prePath = str_replace($this->parent->dataUrl, '', $img->url);
			$tmpArr = explode( '/', str_replace('\\', '/', $prePath) );
			array_pop($tmpArr);
			$folder = implode($tmpArr) . (count($tmpArr)>0?'/':'');
			$path = $this->parent->dataDir . $prePath;
			$res = $ImProc->thumbnailImage($path, $img->modes, $img->width);
			$thumbs = array();
			foreach ($res as $mode => $thumb)
			{
				$thumbs[$mode] = $thumb['error']?$thumb['error']:$this->parent->dataUrl . $folder . $thumb['thumb'];
			} 
			$processed[] = array('id' => $img->id, 'url' => $img->url, 'thumbnails' => $thumbs);
		}
		echo $json->encode(array('status' => 'ok' , 'images' => $processed));
		exit();
	}
	/*
	 * save Gallery info in PHP session
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function saveSession ()
	{
		$this->parent->_init();
		global $user_ID;
		get_currentuserinfo();
		$json = new PSGalleryServicesJSON();
		if (isset($_POST['datastr']))
		{
			update_usermeta($user_ID, 'PSGalleryDataString', $_POST['datastr']);
			echo 'status = ' . $json->encode(array('status' => 'ok'));
		} else
		{
			echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'No data provided'));
		}
		exit(); //need to exit because of the studip die('0') in WP's admin-ajax.php that breaks JSON
	}
	/*
	 * load original image file from URL and create thumnail
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function loadOriginal ()
	{
		$this->parent->_init();
		$json = new PSGalleryServicesJSON();
		$file = $_POST['url'];
		//$source = $_POST['source'];
		$fname = array_pop(explode('/', $file));
		if (! @copy($file, $this->parent->dataDir . $fname))
		{
			echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'Error copying remote file.'));
			exit();
		}
		$_POST['fname'] = $fname;
		$this->registerImage();
	}
	/*
	 * delete image file and record in DB
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function deleteImage ($id = null, $thumbs = null)
	{
		$this->parent->_init();
		$json = new PSGalleryServicesJSON();
		if (! isset($id))
		{
			$id = $_POST['id'];
			$thumbs = $_POST['thumbnails'];
			$doexit = true;
		}
		if (! isset($id))
		{
			echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'No image ID specified.'));
			exit();
		}
		//delete DB record
		$this->parent->wpdb->query('DELETE FROM ' . $this->parent->wpdb->prefix . 'psimagepages WHERE id = ' . $id);
		//delete images
		if (is_array($thumbs))
		{
			foreach ($thumbs as $thumb)
			{
				if($thumb)
				{
					$shortPath = str_replace($this->parent->dataUrl, '', $thumb);
					if (file_exists($this->parent->dataDir . $shortPath))
					{
						unlink($this->parent->dataDir . $shortPath);
					}
				}
			}
		}
		//if Ajax call - output 'ok' response
		if($doexit)
		{
			echo 'status = ' . $json->encode(array('status' => 'ok'));
			exit();
		}
	}
	/*
	 * save Gallery into DB table
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function saveGallery ()
	{
		$this->parent->_init();
		$json = new PSGalleryServicesJSON();
		$gid = intval($_POST['gid']);
		$TblImages = $this->parent->wpdb->prefix . 'psimagepages';
		$TblGalleries = $this->parent->wpdb->prefix . 'psgalleries';
		$datastr = $_POST['datastring'];
		if(!$datastr)
		{
			echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'No data passed for saving. Please try again.'));
			exit();
		}
		$gal = $json->decode(stripcslashes(str_replace('Gallery = ', '', $datastr))); //Do not unescape url!!! File should be saved escaped
		$gTitle = rawurldecode($gal->title);
		if ($gid > 0)
		{
			$rec = $this->parent->wpdb->get_row('SELECT * FROM ' . $TblGalleries . ' WHERE id = "' . $gid . '";');
			$oldTitle = $rec->title; //get old title of gallery
			$code  = $rec->code;
			$created = $rec->created;
		} else
		{
			$created = time();
			$this->parent->wpdb->show_errors();
			$this->parent->wpdb->query($this->parent->wpdb->prepare('INSERT INTO ' . $TblGalleries . ' SET title=%s, created=%d;', $gTitle, $created));
			$gid = $this->parent->wpdb->insert_id;
			$code = $this->_generateGalleryCode($gid);
			$this->parent->wpdb->query($this->parent->wpdb->prepare('UPDATE ' . $TblGalleries . ' SET code = %s WHERE id = %d;', $code, $gid));
		}
		$this->parent->wpdb->query($this->parent->wpdb->prepare('UPDATE ' . $TblGalleries . ' SET title = %s WHERE id = %d;', $gTitle, $gid));
		//clean up all deleted images
		if ($_POST['destrIms'])
		{
			foreach ($_POST['destrIms'] as $img)
			{
				if($img['id'])
				{
					$this->deleteImage($img['id'], $img['thumbnails']);
				}
			}
			//TODO add cleanup of uploaded but not saved to any gallery images
		}
		if ($gal->ims)
		{
			$images = array();
			if (isset($oldTitle)) //rename directory if old exists and title changed
			{
				$oldDir = $this->_getImageSavePath($oldTitle, $gid, true);
				$newDir = $this->_getImageSavePath($gTitle, $gid);
				echo '//renaming ' . $oldDir[name] . ' to ' . $newDir['name'] . "\r\n";
				if ($oldDir['path'] != $newDir['path'] && is_dir($oldDir['path']))
				{
					rmdir($newDir['path']);
					if (! rename($oldDir['path'], $newDir['path']))
					{
						echo 'status = ' . $json->encode(array('status' => 'fail' , 'message' => 'Error updating images directory name.'));
						exit();
					}
					$renamed = true;
				}
			} else
			{
				$newDir = $this->_getImageSavePath($gTitle, $gid);
			}
			$i = 1;
			$images = array();
			$imgIds = array();
			foreach ($gal->ims as $key => $img)
			{
				if(!$img->StockMode)
				{
					$img->src = rawurldecode($img->src);
					$origName = array_pop(explode('/', $img->src));
					$fileDir = str_replace(array('SQ_' . $origName , $origName), array('' , ''), str_replace($this->parent->dataUrl, '', $img->src));
					$orig = $this->parent->dataDir . $fileDir . $origName;
					if ($renamed && ! file_exists($orig))
					{
						$orig = $newDir['path'] . '/' . $origName;
					}
					$img->thumbnails->original = $img->src;
					$fileUri = $this->parent->dataUrl . $newDir['name'] . '/';
					foreach ($img->thumbnails as $mode => $thumb)
					{
						$thumbName = array_pop(explode('/', $thumb));
						if (file_exists($this->parent->dataDir . $thumbName))
						{
							echo '//moving ' . $thumbName . ' to ' . $newDir['name'] . "\r\n";
							if (file_exists($newDir['path'] . '/' . $thumbName))
							{
								unlink($newDir['path'] . '/' . $thumbName);
							}
							rename($this->parent->dataDir . $thumbName, $newDir['path'] . '/' . $thumbName);
						}
						if (file_exists($newDir['path'] . '/' . $thumbName))
						{
							$img->thumbnails->{$mode} = $fileUri . $thumbName;
						}
					}
					$img->src = $img->thumbnails->original;
				}
				//add image page record into DB
				$this->parent->wpdb->query( $this->parent->wpdb->prepare(
					'UPDATE ' . $TblImages . ' SET gal_id=%d, sequence=%d, title=%s, description=%s, image_url=%s WHERE id=%d;'
					, $gid, $i, rawurldecode($img->title), rawurldecode($img->descr), urldecode($img->src), $img->id));
				$images[] = array('id' => $img->id , 'pid' => $i , 'thumbnails' => $img->thumbnails);
				$imgIds[] = $img->id;
				$gal->ims[$key] = $img; //reset the modified element
				$i ++;
			}
		}
		//clean up DB
		$this->parent->wpdb->query('DELETE FROM ' . $TblImages . ' WHERE gal_id=' . $gid . ' AND id NOT IN (' . implode(',', $imgIds) . ');');
		//Save gallery file
		$gal->gid = $gid;
		$gal->code = $code;
		$gal->dateup = time();
		$datastr = 'Gallery = ' . $json->encode($gal);
		$data_filepath = $this->parent->galleryDir . $code . '.json';
		$fp = @fopen($data_filepath, 'wb');
		fwrite($fp, $datastr);
		fclose($fp);
		global $user_ID;
		get_currentuserinfo();
		update_usermeta($user_ID, 'PSGalleryDataString', $datastr);
		echo 'status = ' . $json->encode(array('status' => 'ok' , 'gid' => $gid , 'updated' => time() , 'code' => $code , 'images' => $images));
		exit();
	}
	/*
	 * get/create images directory assiciated with the gallery
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _getImageSavePath ($title, $id = null, $dontCreate = false)
	{
		$this->parent->_init();
		$title = substr($title, 0, 50);
		$title = preg_replace(array('@[\\/\:\*"<>\|\'\?]@' , '@\s+@'), array('' , '_'), stripslashes($title));
		$dirName = $title . ($id ? '_' . $id : '');
		$dir = $this->parent->dataDir . $dirName;
		if (! is_dir($dir) && ! $dontCreate)
		{
			if (! @mkdir($dir))
			{
				return false;
			}
		}
		return array('name' => $dirName , 'path' => $dir);
	}
	/*
	 * generate gallery token to use in embed codes
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function _generateGalleryCode($id)
	{
		$token = base_convert($id, 10, 35);
		for ($i = 0; $i < 8; $i ++)
		{
			$token .= base_convert(rand(0, 35), 10, 35);
		}
		return $token;
	}
	/*
	 * save settings from the setting page into WP options table
	 * 
	 * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
	 */
	function saveSettings()
	{
		$this->parent->_init();
		if(!$_POST['IsPages'] && !$_POST['IsAjax'] && !$_POST['IsShowPL'])
		{
			$_POST['IsPages'] = true;
		}
		$this->parent->params->Settings['Width'] = intval($_POST['Width']);
		$this->parent->params->Settings['Height'] = intval($_POST['Height']);
		$this->parent->params->Settings['IsShowEmail'] = $_POST['IsShowEmail']==='true'?true:false;
		$this->parent->params->Settings['IsShowFB'] = $_POST['IsShowFB']==='true'?true:false;
		$this->parent->params->Settings['IsShowPL'] = $_POST['IsShowPL']==='true'?true:false;
		$this->parent->params->Settings['IsPages'] = $_POST['IsPages']==='true'?true:false;
		$this->parent->params->Settings['IsAjax'] = $_POST['IsAjax']==='true'?true:false;
		$this->parent->params->Settings['IsInline'] = $_POST['IsInline']==='true'?true:false;
		$this->parent->params->Settings['StockImages'] = $_POST['StockImages'];
		$this->parent->params->Settings['IsSavedOnce'] = true;
		update_option('PSGalleryParameters', serialize($this->parent->params));
		if(WP_CACHE===true)
		{
			wp_cache_set('PSGalleryParameters', serialize($this->parent->params), 'Picturesurf');
		}
		echo 'status = {"status": "ok"}';
		exit();
	}
}