<?php
/**
 * Picturesurf.org
 *
 * @copyright	Copyright (c) 2008, Picturesurf.org.
 * @since		Version 1.0
 * @filesource
 */
/**
 * Image processing class
 * This class stands for images manipulations
 *
 * @package		system\application\libraries
 * @author		Denis Zenkovich <denis.zenkovich@picturesurf.org>
 */
class ImageProcessor	
{    
    /**
     * Default constructor
     */
    function ImageProcessor()
	{
	}
    /**
     * Generate a thumbnail
     * 
     * @param $file
     * @param $modes
     * @param $width
     *
     * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
     */    
	function thumbnailImage($file, $modes, $width)
	{
		$result = array();
		$cropModes = array('ST', 'P', 'T', 'LS');
		foreach($modes as $mode)
		{
			$crop = in_array($mode, $cropModes);
			if($mode == 'custom')
			{
				$result[$mode] = $this->createThumbnail($file, $mode, $crop, 402, array($width, 99999999));
			}
			else
			{
				$result[$mode] = $this->createThumbnail($file, $mode, $crop);
			}
		}
		return $result;
	}
    /**
     * The actual thumbnail, this is the actual work horse
     * 
     * @param $fname
     * @param $mode
     * @param $crop
     * @param $minWidth
     * @param $custDim
     *    
     * @return $img
     *
     * @author Denis Zenkovich <denis.zenkovich@picturesurf.org>
     */
	function createThumbnail($fname, $mode = 'T', $crop = true, $minWidth = null, $custDim = null)
	{
		$imageSizes = array
		(
			'ST' => array(72 , 72),
			'T' => array(85 , 85),
			'P' => array(100 , 75),
			'LS' => array(120 , 90),
			'M' => array(600 , 420),
			'SQ' => array(400 , 300)
		);
		if($mode == "custom")
		{
			$imageSizes["custom"] = $custDim;
		}
		if(!isset($imageSizes[$mode]))
		{
			$resp['error'] = "Incorrect thumbnail type specified.";
			return $resp;
		}
		$fname = str_replace('\\', '/', $fname);
		list($w, $h, $type) = getimagesize($fname);
		if($type == 1)
		{
			$img = @imagecreatefromgif($fname);
		} elseif ($type == 2)
		{
			$img = @imagecreatefromjpeg($fname);
		} elseif ($type == 3)
		{
			$img = @imagecreatefrompng($fname);
		}
		if(!$img)
		{
			$resp['error'] = 'Oops! We can not process the image you\'ve uploaded. 
			Please use any image editor program to save the image to "jpg" or "gif" format, and try uploading again.';
			return $resp;
		}
		$pathParts = explode('/', $fname);
		$oName = array_pop($pathParts);
		$path = implode('/', $pathParts);
		$tpath = $path.'/'.$mode.'_'.$oName;
		if(file_exists($tpath)) //if file already there 
		{
			if($mode != "custom") //and is not custom type
			{
				$resp['thumb'] = $mode.'_'.$oName;
				return $resp; //return existing file
			}
			unlink($tpath); //or clean up before new one
		}
		if(!$img)
		{
			$resp['error'] = "System unable to load image.";
			return $resp;
		}
		list($tw, $th) = $imageSizes[$mode];
		if($crop)
		{
			$isBordered = false;
			$ofl = 0;
			$oft = 0;
			$nw = $tw;
			$nh = $th;
			$cw = ($w > $h*($tw/$th)?floor( $h*($tw/$th) ):$w);
			$ch = ($h > $w*($th/$tw)?floor( $w*($th/$tw) ):$h);
			if($w < $tw)
			{
				$ofl = ($tw-$w)/2;
				$nw = $cw = $w;
				$isBordered = true;
			}
			if($h < $th)
			{
				$oft = ($th-$h)/2;
				$nh = $ch = $h;
				$isBordered = true;
			}
			$left = floor( ($w-$cw)/2 );
			$top = floor( ($h-$ch)/2 );
			$thumb = imagecreatetruecolor($tw, $th);
			if($isBordered)
			{
				$white = imagecolorallocate($thumb, 0xFF, 0xFF, 0xFF);
				imagefill($thumb, 0, 0, $white);	// sets background to grey
			}
			imagecopyresampled($thumb, $img, $ofl, $oft, $left, $top, $nw, $nh, $cw, $ch);
		}
		else
		{
			$s = max(array($w/$tw, $h/$th));
			if($s < 1) //if image is smaller then needed - just leave it as it is
			{
				$thumb = imagecreatetruecolor($w, $h);
				imagecopyresampled($thumb, $img, 0, 0, 0, 0, $w, $h, $w, $h);
			}
			else 
			{
				$thumb = imagecreatetruecolor($w/$s, $h/$s);				
                imagecopyresampled($thumb, $img, 0, 0, 0, 0, $w/$s, $h/$s, $w, $h);
			}
		}
		if($type == 1)
		{
			imagegif($thumb, $tpath);
		} 
		elseif ($type == 2)
		{
			imagejpeg($thumb, $tpath, 99);
			$szT = filesize($tpath);
			$szO = filesize($fname);
			if($szT > $szO) //fixup over quality
			{
				imagejpeg($thumb, $tpath, 95-7*round($szT/$szO));
			}
		} 
		elseif ($type == 3)
		{
			imagepng($thumb, $tpath, 2);
		}
		imagedestroy($img);
		imagedestroy($thumb);
		$resp['thumb'] = $mode.'_'.$oName;
		return $resp;
	}
}
?>